@@ -137,6 +137,15 @@ void
 seahorse_util_handle_gpgme (gpgme_error_t err, const char* desc, ...)
 {
     gchar *t = NULL;
+	
+	switch (gpgme_err_code(err)) {
+	case GPG_ERR_CANCELED:
+	case GPG_ERR_NO_ERROR:
+	case GPG_ERR_ECANCELED:
+		return;
+	default:
+		break;
+	}
   
     va_list ap;
     va_start(ap, desc);
@@ -1179,6 +1188,13 @@ seahorse_util_string_equals (const gchar *s1, const gchar *s2)
     return g_str_equal (s1, s2);
 }
 
+void        
+seahorse_util_string_lower (gchar *s)
+{
+    for ( ; *s; s++)
+        *s = g_ascii_tolower (*s);
+}
+
 /* Free a GSList along with string values */
 GSList*
 seahorse_util_string_slist_free (GSList *list)
