@@ -66,7 +66,9 @@ static const struct poptOption options[] = {
     { "uri-list", 'T', POPT_ARG_NONE | POPT_ARG_VAL, &read_uris, TRUE,
       N_("Read list of URIs on standard in"), NULL },
 	
-	{ NULL, '\0', 0, NULL, 0, NULL, NULL }
+	POPT_AUTOHELP
+	
+	POPT_TABLEEND
 };
 
 
@@ -156,6 +158,7 @@ do_import (SeahorseContext *sctx, const gchar **paths)
                                     _("Imported %d keys"), keys);
         dlg = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                              GTK_MESSAGE_INFO, GTK_BUTTONS_OK, t);
+        g_free (t);
         gtk_dialog_run (GTK_DIALOG (dlg));
         gtk_widget_destroy (dlg);
     }
@@ -429,6 +432,11 @@ main (int argc, char **argv)
 	gpgme_set_locale(NULL, LC_MESSAGES, setlocale(LC_MESSAGES, NULL));
 #endif
 
+#ifdef _DEBUG 
+    g_log_set_always_fatal (G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL | 
+                            G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
+#endif
+    
     program = gnome_program_init(PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
                     GNOME_PARAM_POPT_TABLE, options,
                     GNOME_PARAM_HUMAN_READABLE_NAME, _("Encryption Key Manager"),
