@@ -22,8 +22,10 @@
 #include <gnome.h>
 #include <time.h>
 
+#include "seahorse-gpgmex.h"
 #include "seahorse-key-dialogs.h"
 #include "seahorse-key-widget.h"
+#include "seahorse-op.h"
 #include "seahorse-util.h"
 #include "seahorse-key-op.h"
 
@@ -33,8 +35,10 @@
 static gint
 get_subkey_index (SeahorseWidget *swidget)
 {
-    return (gtk_notebook_get_current_page (GTK_NOTEBOOK (
-        glade_xml_get_widget (swidget->xml, NOTEBOOK))) - 1);
+    gint index = gtk_notebook_get_current_page (GTK_NOTEBOOK (
+        glade_xml_get_widget (swidget->xml, NOTEBOOK))) - 3;
+    g_return_val_if_fail (index >= 0, -1);
+    return index;        
 }
 
 
@@ -48,68 +52,16 @@ trust_changed (GtkOptionMenu *optionmenu, SeahorseWidget *swidget)
 	skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
 	
 	if (seahorse_key_get_trust (skey) != trust)
-		seahorse_key_op_set_trust (swidget->sctx, skey, trust);
+		seahorse_key_op_set_trust (skey, trust);
 }
 
 static void
 disabled_toggled (GtkToggleButton *togglebutton, SeahorseWidget *swidget)
 {
-	seahorse_key_op_set_disabled (swidget->sctx, SEAHORSE_KEY_WIDGET (swidget)->skey,
+	seahorse_key_op_set_disabled (SEAHORSE_KEY_WIDGET (swidget)->skey,
 		gtk_toggle_button_get_active (togglebutton));
 }
 
-static void
-primary_expires_date_changed (GnomeDateEdit *gde, SeahorseWidget *swidget)
-{
-    seahorse_key_pair_op_set_expires (swidget->sctx, SEAHORSE_KEY_PAIR (
-        SEAHORSE_KEY_WIDGET (swidget)->skey), 0, gnome_date_edit_get_time (gde));
-}
-
-static void
-subkey_expires_date_changed (GnomeDateEdit *gde, SeahorseWidget *swidget)
-{
-    seahorse_key_pair_op_set_expires (swidget->sctx, SEAHORSE_KEY_PAIR (
-        SEAHORSE_KEY_WIDGET (swidget)->skey), get_subkey_index (swidget),
-        gnome_date_edit_get_time (gde));
-}
-
-static void
-set_date_edit_sensitive (GtkToggleButton *togglebutton, GtkWidget *widget)
-{
-    gtk_widget_set_sensitive (widget, !gtk_toggle_button_get_active (togglebutton));
-}
-
-static void
-never_expires_primary_toggled (GtkToggleButton *togglebutton, SeahorseWidget *swidget)
-{
-    SeahorseKey *skey;
- 
-    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
-    
-    /* if want to never expire & expires, set to never expires */
-    if (gtk_toggle_button_get_active (togglebutton) && skey->key->subkeys->expires) {
-        seahorse_key_pair_op_set_expires (swidget->sctx, SEAHORSE_KEY_PAIR (skey), 0, 0);
-    }
-}
-
-static void
-never_expires_subkey_toggled (GtkToggleButton *togglebutton, SeahorseWidget *swidget)
-{
-    SeahorseKey *skey;
-    gint index;
-    gpgme_subkey_t subkey;
- 
-    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
-    index = get_subkey_index (swidget);
-
-    subkey = seahorse_key_get_nth_subkey (skey, index);
-    
-    /* if want to never expire & expires, set to never expires */
-    if (gtk_toggle_button_get_active (togglebutton) && subkey->expires) {
-        seahorse_key_pair_op_set_expires (swidget->sctx, SEAHORSE_KEY_PAIR (skey), index, 0);
-    }
-}
-
 static void
 do_stat_label (const gchar *label, GtkTable *table, guint left, guint top,
 	       gboolean selectable, GtkTooltips *tips, const gchar *tip)
@@ -159,11 +111,96 @@ do_stat_button (gchar *label, const gchar *stock_id)
 	return container;
 }
 
+static void
+key_property_labels (SeahorseWidget *swidget)
+{
+    gchar dbuffer[G_ASCII_DTOSTR_BUF_SIZE];
+    gpgme_subkey_t subkey;
+    SeahorseKey *skey;
+    GtkWidget *w;
+    const gchar *label;
+    gchar *t, *x;
+
+    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    subkey = skey->key->subkeys;
+    
+    w = glade_xml_get_widget (swidget->xml, "keyid");
+    label = seahorse_key_get_keyid (skey, 0); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "fingerprint");
+    label = seahorse_key_get_fingerprint (skey);  
+    gtk_label_set_text (GTK_LABEL (w), label);
+
+    w = glade_xml_get_widget (swidget->xml, "label_type");
+    label = gpgme_pubkey_algo_name(subkey->pubkey_algo);
+
+    if (label == NULL)
+        label = _("Unknown");
+    else if (g_str_equal ("ElG", label))
+        label = _("ElGamal");
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_created");
+    label = seahorse_util_get_date_string (subkey->timestamp); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+
+    w = glade_xml_get_widget (swidget->xml, "label_strength");
+    g_ascii_dtostr(dbuffer, G_ASCII_DTOSTR_BUF_SIZE, subkey->length);
+    gtk_label_set_text (GTK_LABEL (w), dbuffer);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_expires");
+    label = seahorse_util_get_date_string (subkey->expires);
+    if (g_str_equal("0", label))
+	    label = _("Never");
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    w = glade_xml_get_widget (swidget->xml, "label_created");
+    label = seahorse_util_get_date_string (subkey->timestamp); 
+    gtk_label_set_text (GTK_LABEL (w), label);
+    
+    if ((t = seahorse_key_get_userid_name (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_name");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+    
+    if ((t = seahorse_key_get_userid_email (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_email");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+    
+    if ((t = seahorse_key_get_userid_comment (skey, 0)) != NULL) {
+        w = glade_xml_get_widget (swidget->xml, "label_comment");
+        gtk_label_set_text (GTK_LABEL (w), t);  
+        g_free (t);
+    }
+
+    t = seahorse_key_get_userid (skey, 0);
+    x = g_strdup_printf (_("%s Properties"), t);
+    g_free (t);
+    
+    w = glade_xml_get_widget (swidget->xml, swidget->name);
+    gtk_window_set_title (GTK_WINDOW (w), x);
+    g_free (x);
+}
+
+static void
+expiry_clicked (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    SeahorseKey *skey;
+    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    
+    if (SEAHORSE_IS_KEY_PAIR (skey))
+        seahorse_expires_new(swidget->sctx, skey, get_subkey_index(swidget));
+}
+
 static void
 do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgme_subkey_t subkey)
 {
     SeahorseKey *skey;
-    GtkWidget *widget, *date_edit;
+    GtkWidget *widget;
     GtkTooltips *tips;
     const gchar *str;
     gchar * buf;
@@ -179,7 +216,9 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
     /* type */
     do_stat_label (_("Type:"), table, 2, top, FALSE, tips, _("Algorithm"));
     str = gpgme_pubkey_algo_name(subkey->pubkey_algo);
-    if (g_str_equal ("ElG", str))
+    if (str == NULL)
+        str = "Unknown";
+    else if (g_str_equal ("ElG", str))
         str = "ElGamal";
     do_stat_label (str, table, 3, top, FALSE, NULL, NULL);
     /* created */
@@ -200,71 +239,25 @@ do_stats (SeahorseWidget *swidget, GtkTable *table, guint top, guint index, gpgm
         str = _("Expired");
     do_stat_label (str, table, 1, top+3, FALSE, NULL, NULL);
     /* expires */
-    do_stat_label (_("Expiration Date:"), table, 0, top+2, FALSE, NULL, NULL);
+    do_stat_label (_("Expires:"), table, 0, top+2, FALSE, NULL, NULL);
  
-   if (!SEAHORSE_IS_KEY_PAIR (skey)) {
-        if (subkey->expires)
-            do_stat_label (seahorse_util_get_date_string (subkey->expires),
-                table, 1, top+2, FALSE, NULL, NULL);
-        else
-            do_stat_label (_("Never Expires"), table, 1, top+2, FALSE, NULL, NULL);
-   } else {
-        date_edit = gnome_date_edit_new (subkey->expires, FALSE, FALSE);
-        gtk_widget_show (date_edit);
-        gtk_table_attach (table, date_edit, 1, 2, top+2, top+3, GTK_FILL, 0, 0, 0);
-        gtk_widget_set_sensitive (date_edit, subkey->expires);
-     
-        if (index == 0)
-            g_signal_connect_after (GNOME_DATE_EDIT (date_edit), "date_changed",
-                G_CALLBACK (primary_expires_date_changed), swidget);
-        else
-            g_signal_connect_after (GNOME_DATE_EDIT (date_edit), "date_changed",
-                G_CALLBACK (subkey_expires_date_changed), swidget);
-        
-        widget = gtk_check_button_new_with_mnemonic (_("Never E_xpires"));
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), !subkey->expires);
-        gtk_tooltips_set_tip (tips, widget, _("If key never expires"), NULL);
+    if (subkey->expires)
+        do_stat_label (seahorse_util_get_date_string (subkey->expires),
+                       table, 1, top+2, FALSE, NULL, NULL);
+    else
+        do_stat_label (_("Never"), table, 1, top+2, FALSE, NULL, NULL);
+    
+    if (SEAHORSE_IS_KEY_PAIR (skey)) {
+        widget = gtk_button_new_with_mnemonic(_("Change Expiry Date"));
+        gtk_button_set_relief (GTK_BUTTON (widget), GTK_RELIEF_HALF);
         gtk_widget_show (widget);
-        gtk_table_attach (table, widget, 2, 3, top+2, top+3, GTK_FILL, 0, 0, 0);
+        gtk_table_attach (table, widget, 2, 3, top + 2, top + 3, GTK_FILL, 0, 0, 0);
        
-        g_signal_connect_after (GTK_TOGGLE_BUTTON (widget), "toggled",
-            G_CALLBACK (set_date_edit_sensitive), date_edit);
-      
-        if (index == 0)
-            g_signal_connect_after (GTK_TOGGLE_BUTTON (widget), "toggled",
-                G_CALLBACK (never_expires_primary_toggled), swidget);
-        else
-            g_signal_connect_after (GTK_TOGGLE_BUTTON (widget), "toggled",
-                G_CALLBACK (never_expires_subkey_toggled), swidget);
+        g_signal_connect_after (GTK_BUTTON (widget), "clicked", 
+                                G_CALLBACK (expiry_clicked), swidget);      
     }
-}
-
-static void
-do_key_info (SeahorseWidget *swidget)
-{
-    SeahorseKey *skey;
-    GtkWidget *w;
-    gchar *t, *x;
-
-    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
-    
-    w = glade_xml_get_widget (swidget->xml, "fingerprint");
-    t = seahorse_key_get_fingerprint (skey); 
-    gtk_label_set_text (GTK_LABEL (w), t);
-    g_free (t);
-    
-    w = glade_xml_get_widget (swidget->xml, "userid");
-    t = seahorse_key_get_userid (skey, 0); 
-    gtk_label_set_text (GTK_LABEL (w), t);
-    
-    x = g_strdup_printf (_("%s Properties"), t);
-    g_free (t);
-    
-    w = glade_xml_get_widget (swidget->xml, swidget->name);
-    gtk_window_set_title (GTK_WINDOW (w), x);
-    g_free (x);
-}
     
+}    
 
 static void
 passphrase_clicked (GtkWidget *widget, SeahorseWidget *swidget)
@@ -273,7 +266,41 @@ passphrase_clicked (GtkWidget *widget, SeahorseWidget *swidget)
     skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
     
     if (SEAHORSE_IS_KEY_PAIR (skey))
-        seahorse_key_pair_op_change_pass (swidget->sctx, SEAHORSE_KEY_PAIR (skey));
+        seahorse_key_pair_op_change_pass (SEAHORSE_KEY_PAIR (skey));
+}
+
+static void
+export_clicked (GtkWidget *widget, SeahorseWidget *swidget)
+{
+    SeahorseKey *skey;
+    GtkWidget *dialog;
+    gchar* uri = NULL;
+    GError *err = NULL;
+    GList *keys = NULL;
+    
+    skey = SEAHORSE_KEY_WIDGET (swidget)->skey;
+    if (!SEAHORSE_IS_KEY_PAIR (skey))
+        return;
+
+    keys = g_list_prepend (keys, skey);
+    
+    dialog = seahorse_util_chooser_save_new (_("Export Complete Key"), 
+                GTK_WINDOW(seahorse_widget_get_top (swidget)));
+    seahorse_util_chooser_show_key_files (dialog);
+    seahorse_util_chooser_set_filename (dialog, keys);
+    
+    uri = seahorse_util_chooser_save_prompt (dialog);
+    if(uri) {
+        seahorse_op_export_file (keys, TRUE, uri, &err);
+
+        if (err != NULL)
+            seahorse_util_handle_error (err, _("Couldn't export key to \"%s\""),
+                    seahorse_util_uri_get_last (uri));
+
+        g_free (uri);
+    }
+    
+    g_list_free (keys);
 }
 
 static void
@@ -541,10 +568,10 @@ uid_primary_clicked (GtkWidget *widget, SeahorseWidget *swidget)
     
     index = get_selected_uid (swidget);
     if (index >= 1) {
-        err = seahorse_key_op_primary_uid (swidget->sctx, skey, index);
+        err = seahorse_key_op_primary_uid (skey, index);
         
         if (!GPG_IS_OK (err)) 
-            seahorse_util_handle_error (err, _("Couldn't change primary user ID"));
+            seahorse_util_handle_gpgme (err, _("Couldn't change primary user ID"));
     }
 }
 
@@ -653,15 +680,23 @@ static void
 key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseWidget *swidget)
 {
 	switch (change) {
+        case SKEY_CHANGE_ALL:
+            do_uid_list (swidget);
+            key_property_labels (swidget);
+            do_signatures (swidget);
+            do_signature_list (swidget);
+            do_subkeys (swidget);
+            break;
 		case SKEY_CHANGE_UIDS:
 			do_uid_list (swidget);
-            do_key_info (swidget);
+            key_property_labels (swidget);
 			break;
         case SKEY_CHANGE_SIGNERS:
             do_signatures (swidget);
             do_signature_list (swidget);
             break;
-		case SKEY_CHANGE_SUBKEYS: case SKEY_CHANGE_EXPIRES:
+		case SKEY_CHANGE_SUBKEYS: 
+        case SKEY_CHANGE_EXPIRES:
 			do_subkeys (swidget);
 			break;
 		default:
@@ -669,78 +704,126 @@ key_changed (SeahorseKey *skey, SeahorseKeyChange change, SeahorseWidget *swidge
 	}
 }
 
+static void
+key_destroyed (GtkObject *object, SeahorseWidget *swidget)
+{
+    seahorse_widget_destroy (swidget);
+}
+
 static void
 properties_destroyed (GtkObject *object, SeahorseWidget *swidget)
 {
-	g_signal_handlers_disconnect_by_func (SEAHORSE_KEY_WIDGET (swidget)->skey,
-		key_changed, swidget);
+    g_signal_handlers_disconnect_by_func (SEAHORSE_KEY_WIDGET (swidget)->skey,
+                                          key_changed, swidget);
+    g_signal_handlers_disconnect_by_func (SEAHORSE_KEY_WIDGET (swidget)->skey,
+                                          key_destroyed, swidget);
 }
 
+
+
+
 void
 seahorse_key_properties_new (SeahorseContext *sctx, SeahorseKey *skey)
 {
+    SeahorseKeySource *sksrc;
 	SeahorseWidget *swidget;
 	GtkWidget *widget;
+    gboolean remote;
+    
+    remote = seahorse_key_get_loaded_info (skey) == SKEY_INFO_REMOTE;
+    
+    /* Reload the key to make sure to get all the props */
+    sksrc = seahorse_key_get_source (skey);
+    g_return_if_fail (sksrc != NULL);
+        
+    /* Don't trigger the import of remote keys if possible */
+    if (!remote) {
+        /* This causes the key source to get any specific info about the key */
+        seahorse_key_source_refresh_sync (sksrc, seahorse_key_get_id (skey->key));
+        skey = seahorse_key_source_get_key (sksrc, seahorse_key_get_id (skey->key));
+        g_return_if_fail (skey != NULL);                
+    }
 	
 	swidget = seahorse_key_widget_new ("key-properties", sctx, skey);
-	g_return_if_fail (swidget != NULL);
+	
+	/* This happens if the window is already open */
+	if (swidget == NULL)
+		return;
 	
 	widget = glade_xml_get_widget (swidget->xml, swidget->name);
 	g_signal_connect (GTK_OBJECT (widget), "destroy", G_CALLBACK (properties_destroyed), swidget);
-	g_signal_connect_after (skey, "changed", G_CALLBACK (key_changed), swidget);
+    g_signal_connect_after (skey, "changed", G_CALLBACK (key_changed), swidget);
+    g_signal_connect_after (skey, "destroy", G_CALLBACK (key_destroyed), swidget);
 	
-	do_stats (swidget, GTK_TABLE (glade_xml_get_widget (swidget->xml, "primary_table")), 3, 0,
-		skey->key->subkeys);
-     
-    /* Main key info */
-    do_key_info (swidget);
-     
-    /* Change password button */
+	key_property_labels (swidget);
+    
+    /* Secret key stuff */    
     if(SEAHORSE_IS_KEY_PAIR (skey)) {
+        
+        /* Change password button */
         widget = glade_xml_get_widget (swidget->xml, "passphrase");
         gtk_widget_set_sensitive (widget, TRUE);
         glade_xml_signal_connect_data (swidget->xml, "passphrase_clicked",
                     G_CALLBACK (passphrase_clicked), swidget);
+
+        /* Export secret key button */
+        widget = glade_xml_get_widget (swidget->xml, "export_complete");
+        gtk_widget_set_sensitive (widget, TRUE);
+        glade_xml_signal_connect_data (swidget->xml, "export_clicked",
+                    G_CALLBACK (export_clicked), swidget);
     }
 
 	do_subkeys (swidget);
-    
-    /* Uid page */
-    widget = glade_xml_get_widget (swidget->xml, "uid-list");
-    g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), "changed",
-                G_CALLBACK (uid_sel_changed), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_add_clicked",
-                G_CALLBACK (uid_add_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_primary_clicked",
-                G_CALLBACK (uid_primary_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_sign_clicked",
-                G_CALLBACK (uid_sign_clicked), swidget);
-    glade_xml_signal_connect_data (swidget->xml, "uid_delete_clicked",
-                G_CALLBACK (uid_delete_clicked), swidget);
     do_uid_list (swidget);
 
-    
-    /* Signature setup */
-    do_signatures (swidget);
-    do_signature_list (swidget);
-    glade_xml_signal_connect_data (swidget->xml, "sigs_changed",
-          G_CALLBACK (signature_sel_changed), swidget);        
+    /* Disable stuff not appropriate for remote */
+    if (remote) {
+        gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "trust"), FALSE);
+        gtk_widget_set_sensitive (glade_xml_get_widget (swidget->xml, "disabled"), FALSE);
 
-	/* disable trust options */
-	if (SEAHORSE_IS_KEY_PAIR (skey))
-		widget = glade_xml_get_widget (swidget->xml, "unknown");
-	else
-		widget = glade_xml_get_widget (swidget->xml, "ultimate");
-	gtk_widget_set_sensitive (widget, FALSE);
-	
-	gtk_option_menu_set_history (GTK_OPTION_MENU (glade_xml_get_widget (swidget->xml, "trust")),
-		seahorse_key_get_trust (skey) - 1);
+        /* Remove the signatures page (3rd page) */ 
+        widget = glade_xml_get_widget (swidget->xml, NOTEBOOK);
+        gtk_notebook_remove_page (GTK_NOTEBOOK (widget), 2);
+        
+    /* Stuff valid for local keys */
+    } else {     
+
+        /* Uid page */
+        widget = glade_xml_get_widget (swidget->xml, "uid-list");
+        g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (widget)), "changed",
+                    G_CALLBACK (uid_sel_changed), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_add_clicked",
+                    G_CALLBACK (uid_add_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_primary_clicked",
+                    G_CALLBACK (uid_primary_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_sign_clicked",
+                    G_CALLBACK (uid_sign_clicked), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "uid_delete_clicked",
+                    G_CALLBACK (uid_delete_clicked), swidget);
+                    
+        /* Signature setup */
+        do_signatures (swidget);
+        do_signature_list (swidget);
+        glade_xml_signal_connect_data (swidget->xml, "sigs_changed",
+                                       G_CALLBACK (signature_sel_changed), swidget);        
+
+        /* disable trust options */
+        if (SEAHORSE_IS_KEY_PAIR (skey))
+            widget = glade_xml_get_widget (swidget->xml, "unknown");
+        else
+            widget = glade_xml_get_widget (swidget->xml, "ultimate");
+        gtk_widget_set_sensitive (widget, FALSE);
+        
+    	gtk_option_menu_set_history (GTK_OPTION_MENU (glade_xml_get_widget (swidget->xml, "trust")),
+	                                 seahorse_key_get_trust (skey) - 1);
+        glade_xml_signal_connect_data (swidget->xml, "trust_changed",
+                                       G_CALLBACK (trust_changed), swidget);
+        glade_xml_signal_connect_data (swidget->xml, "disabled_toggled",
+                                       G_CALLBACK (disabled_toggled), swidget);
+    }
 	
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (swidget->xml, "disabled")),
-		skey->key->disabled);
-	
-	glade_xml_signal_connect_data (swidget->xml, "trust_changed",
-		G_CALLBACK (trust_changed), swidget);
-	glade_xml_signal_connect_data (swidget->xml, "disabled_toggled",
-		G_CALLBACK (disabled_toggled), swidget);
+		                          skey->key->disabled);
+
+    seahorse_widget_show (swidget);    
 }
